* 1.01: calculate the std error of sig_u, sig_v
* 1.10: Force matrix to be symmetric before inverse.
*       Add [if] [in] options.
* 1.20: Add t and p values, and -cost- option.
* 1.30: Estimate sigma_u^2 instead of sigma_u.
* 2.00: Use -preserve- to avoid switch-data-sign problem.

*! Hung-Jen Wang, 16Nov03.
* Institute of Economics
* Academia Sinica
* hjwang@econ.sinica.edu.tw
* http://www.sinica.edu.tw/~wanghj/

capture program drop sfMoM

program define sfMoM

version 6

syntax varlist [if] [in], OKVARS(varlist) MISVAR(varlist) [COST PRODuction NOSYM]
marksample touse, nov

* nosym: Not forcing the matrix to be symmetric before inverting. Specify this for debugging purpose.
* The model always has a constant. That means, you cannot de-mean all the dependent and independent variables.

preserve /* keep the current copy of the dataset, so that in case of -cost-, do not need to switch sign back when estimation is done or interrupted */


/* --- check whether more than 1 misvar is specified ---- */

local cw : word count `misvar'
if `cw' > 1 {
     di in red "You cannot specify more than one mis-measured variables."
     error 200
}


/* --- determine whether it's a cost or a production frontier --- */



if  (("`cost'"~="") & ("`production'"~="")) {
     di in red "Need to specify one and only one of the -cost- and -production- option."
     error 200
}


/*--- remove collinear independent variables ---- */

tempvar y oldmis
quie gen double `y' = `varlist' if `touse' /* generate additional variable */
local oldmis `misvar'

     * firstly, detect collinear on the specified sample

 _rmcoll `misvar' `okvars' if `touse'
local list `r(varlist)'
tokenize `list'


local misvar `1' /* the first on the list should be the mismeasured */
     if "`misvar'" ~= "`oldmis'" { /* means the mismeasured is dropped due to collinearity */
         di in red "The variable `oldmis' is dropped due to collinearity with other independent variables."
         di in red "   The estimation cannot proceed further."
         error 198
     }
mac shift
local okvars `*' /* the rest of the ok variables */

     * secondly, detect missing observations due to lags, etc.
tempvar useobs
quie reg `y' `okvars' `misvar' if `touse'
quie gen `useobs' = cond(e(sample), 1, 0) /* it is based on touse and the regression */

     * lastly, detect collinear again on the reduced sample

 _rmcoll `misvar' `okvars' if `useobs' == 1
local list `r(varlist)'
tokenize `list'
local misvar `1' /* the first on the list should be the mismeasured */
     if "`misvar'" ~= "`oldmis'" { /* means the mismeasured is dropped due to collinearity */
         di in red "The variable `oldmis' is dropped due to collinearity with other independent variables."
         di in red "   The estimation cannot proceed further."
         error 198
     }
mac shift
local okvars `*' /* the rest of the ok variables */



***********************
  /* --- flip data if estimate cost frontier ------- */

if "`cost'" ~= "" { /* estimate a cost frontier */

   foreach X of local okvars  {
     quie replace `X' = -`X'
   }
     quie replace `misvar' = -`misvar'

     quie replace `y' = -`y'

}

*********************

quie sum `useobs' if `useobs'==1, meanonly
local nofobs = r(N)

tokenize `okvars'
local kk = 1
while "`1'"~=""{
   local xvar`kk' `1'
   local kk = `kk'+1
   mac shift
}


tempvar z e x  ye ze seq

quie gen `seq' = _n if `useobs' == 1

quie gen double `z' = `misvar' if `useobs' == 1


quie reg `y' `okvars' if `useobs'==1
quie predict `ye' if e(sample), resid
  scalar muy0 = _b[_cons]
  mat b0 = e(b)

quie reg `z' `okvars' if `useobs'==1
quie predict `ze' if e(sample), resid
  scalar muz0 = _b[_cons]
  mat b1 = e(b)


tempvar yesum zesum ye2sum yezesum ze2sum ye2zesm yeze2sm ye3sum

quie egen double `yesum' = mean(`ye') if `useobs'==1
quie egen double `zesum' = mean(`ze') if `useobs'==1
quie egen double `ye2sum' = mean((`ye')^2) if `useobs'==1
quie egen double `yezesum' = mean((`ye')*(`ze')) if `useobs'==1
quie egen double `ze2sum' = mean((`ze')^2) if `useobs'==1
quie egen double `ye2zesm' = mean(((`ye')^2)*(`ze')) if `useobs'==1
quie egen double `yeze2sm' = mean(((`ze')^2)*(`ye')) if `useobs'==1
quie egen double `ye3sum' = mean((`ye')^3) if `useobs'==1

 /* estimate beta; z's coefficient */

sort `ye2sum' `ze2sum' /* so that the first obs won't have missing values */
assert `ye2sum'[1]~=.
assert `ze2sum'[1]~=.

mat gb = (`ye2sum'[1] \ `yezesum'[1] \ `ze2sum'[1] \ `ye2zesm'[1] \ `yeze2sm'[1] \ `ye3sum'[1]) /* to be used late */

scalar betahat = `ye2zesm'[1]/`yeze2sm'[1]
scalar Ezse2 = `yezesum'[1]/betahat
scalar Ezse3 = `yeze2sm'[1]/betahat
scalar Ee2   = `ze2sum'[1]- Ezse2
scalar sigu2 = ((`ye3sum'[1] - ((betahat)^3)*(Ezse3))/(sqrt(2/_pi)*(1 - 4/_pi)))^(2)
scalar sigu2 =  (sigu2)^(1/3)
scalar sigv2 = `ye2sum'[1] - ((betahat)^2)*(Ezse2) - (1-2/_pi)*(sigu2)
scalar sigv = sqrt(sigv2)


if scalar(sigv) == . {
     di " "
     di in red "There are problems estimating sigma_v^2 for this model; it returns a negative value."
     di in red " (The method of moment approach does not guarantee a positive sigma_v^2.)"
     di in red " The estimation cannot proceed. Please try other variable specifications."

     error 200
}


/* estimate the intercept and gamma (x's coefficient) */

local nofvars : word count `okvars'

scalar alphahat = muy0 - betahat*muz0 + sqrt(2/_pi)*(sqrt(sigu2))
* scalar alphahat2 = muy0 - betahat*muz0 + sqrt(2/_pi)*(sigu) + betahat*sqrt(Ee2)

if "`cost'" ~= "" { /* means a cost frontier; need to flip the sign of the constant */
  scalar alphahat = -1*scalar(alphahat)
}


**** covariance matrix

** matrix of C


mat C = ( 2*betahat*Ezse2, betahat^2, 0, 0, 1, (1-2/_pi) \ /*
     */   Ezse2, betahat, 0, 0, 0, 0 \ /*
     */   0, 1, 0, 1, 0, 0 \ /*
     */   2*betahat*Ezse3, 0, betahat^2, 0 , 0, 0 \ /*
     */   Ezse3, 0, betahat, 0, 0, 0 \ /*
     */   3*betahat^2*Ezse3, 0, betahat^3, 0, 0, 1.5*sqrt(2/_pi)*(1-4/_pi)*(sqrt(sigu2)) )


**** big G bar

local nofc = (`nofvars'+1)*2 /* number of columns */

mat Gbar = J(6, `nofc', 0) /* a zero 6Xnofc matrix */

tempvar zeye
quie gen double `zeye' = `ze'*`ye' if `useobs' == 1 /* for later use */

     /* the 4th and the 5th row */


tempvar tem1
quie egen double `tem1' = mean(-2*`zeye') if `useobs'==1
sort `tem1'
mat Gbar[4,1] = `tem1'[1]
mat Gbar[5, `nofvars'+1+1] = `tem1'[1]
drop `tem1'


forvalues jj = 1/`nofvars' {
     tempvar tem1
     quie egen double `tem1' = mean(-2*(`xvar`jj'')*`zeye') if `useobs'==1
     sort `tem1'
     mat Gbar[4, 1+`jj'] = `tem1'[1]
     mat Gbar[5, 1+`nofvars'+ 1+`jj']=`tem1'[1]
     drop `tem1'
}

tempvar tem1
quie egen double `tem1' = mean(-(`ye')^2) if `useobs'==1
sort `tem1'
mat Gbar[4, 1+`nofvars'+1] = `tem1'[1]
drop `tem1'

forvalues jj = 1/`nofvars'{
     tempvar tem1
     quie egen double `tem1' = mean(-(`xvar`jj'')*(`ye')^2) if `useobs'==1
     sort `tem1'
     mat Gbar[4, `nofvars'+1 + `jj'+1] = `tem1'[1]
     drop `tem1'
}

    /* the remaining of the 5th row */

tempvar tem1
quie egen double `tem1' = mean(-(`ze')^2) if `useobs'==1
sort `tem1'
mat Gbar[5, 1] = `tem1'[1]
drop `tem1'
forvalues jj = 1/`nofvars'{
     tempvar tem1
     quie egen double `tem1' = mean(-(`xvar`jj'')*(`ze')^2) if `useobs'==1
     sort `tem1'
     mat Gbar[5, 1+`jj'] = `tem1'[1]
     drop `tem1'
}


   /* the 6th row */

tempvar tem1
quie egen double `tem1' = mean(-3*(`ye')^2) if `useobs'==1
sort `tem1'
mat Gbar[6,1]=`tem1'[1]
drop `tem1'
forvalues jj = 1/`nofvars' {
     tempvar tem1
     quie egen double `tem1' = mean(-3*(`xvar`jj'')*(`ye')^2) if `useobs' == 1
     sort `tem1'
     mat Gbar[6, 1+`jj']=`tem1'[1]
     drop `tem1'
}


** matrix of Omega

   /* g_i */

capture drop `zeye'

tempvar ye2 zeye ze2 zeye2 ze2ye ye3
quie gen double `ye2' = (`ye')^2 if `useobs'==1
quie gen double `zeye' = (`ze')*(`ye') if `useobs'==1
quie gen double `ze2' = (`ze')^2 if `useobs'==1
quie gen double `zeye2' = (`ze')*(`ye')^2 if `useobs'==1
quie gen double `ze2ye' = ((`ze')^2)*(`ye') if `useobs'==1
quie gen double `ye3' = (`ye')^3 if `useobs'==1

forvalues jj = 1/`nofvars' {
  tempvar x`jj'ye x`jj'ze
  quie gen double `x`jj'ye' = (`xvar`jj'')*(`ye') if `useobs'==1
  quie gen double `x`jj'ze' = (`xvar`jj'')*(`ze') if `useobs'==1
}

tempvar mycon
quie gen `mycon' = 1 if `useobs' == 1


quie mat accum XX0 = `mycon' `okvars' if `useobs'==1, nocons
mat XX1 = XX0/`nofobs'


mat idenI2 = (1, 0 \ 0, 1)


mat omega = J(6,6,0)

gsort -`useobs' /* so that valid obs are on the top */

forvalues jj = 1/`nofobs' { /* loop over observations */
   mat gi  = (`ye2'[`jj'] \ `zeye'[`jj'] \ `ze2'[`jj'] \ `zeye2'[`jj'] \ `ze2ye'[`jj'] \ `ye3'[`jj'])

     /* the R_i(mu) vector; (k+1)*2X1 for each obs */
   mat Ri = J((`nofvars'+1)*2, 1, 0) /* a column vector */
   mat Ri[1,1] = `ye'[`jj']
   forvalues kk = 1/`nofvars' {
        mat Ri[1+`kk',1] = `x`kk'ye'[`jj']
   }

   mat Ri[1+`nofvars'+1, 1] = `ze'[`jj']
   forvalues kk = 1/`nofvars' {
        mat Ri[1+`nofvars'+ 1 + `kk', 1] = `x`kk'ze'[`jj']
   }

     /* the ph_mu_i matrix */

   tempname tempm

   mat `tempm' = idenI2 # XX1
   if "`nosym'" == "" {
     mat `tempm' = 0.5*(`tempm' + `tempm'') /* force it to be symmetric */
   }
   mat phii = syminv(`tempm')*Ri

   mat omega = omega + (gi-gb + Gbar*phii)*(gi-gb+Gbar*phii)'

} /* forvalues jj */

mat omega = omega/(`nofobs')
   if "`nosym'" == "" {
      mat omega = 0.5*(omega + omega') /* force it to be symmetric */
    }
tempname tempm2
mat `tempm2' = C'*syminv(omega)*C
   if "`nosym'" == "" {
     mat `tempm2' = 0.5*(`tempm2' + `tempm2'') /* force it to be symmetric */
    }
mat asymcov = syminv(`tempm2')/(`nofobs')

scalar betastd = sqrt(asymcov[1,1])
scalar v2std = sqrt(asymcov[5,5]) /* std dev. of sigma_v^2 */
scalar u2std =  sqrt(asymcov[6,6])  /* std dev. of sigma_u^2 */

 /* -- use delta method to get the std dev. of sigma_v -- */

scalar vstd = sqrt(asymcov[5,5]*0.25*(sigv)^(-2)) /* std dev. of sigma_v */


forvalues jj = 1/`nofvars' {
  scalar gamma`jj' = b0[1,`jj'] - betahat*b1[1,`jj']
}


mat muz1 = b1[1, 1..`nofvars']

** mat mystry = J(6,6,0)
mat mystry = J(2*(1+`nofvars')+2, 2*(1+`nofvars')+2, 0)
** mat mystry2 = J(5,5,0)
mat mystry2 = J(2*(1+`nofvars')+1, 2*(1+`nofvars')+1, 0)

mat tem1 = J(1, `nofvars', 0)
mat Gbar2 = (1, tem1, -betahat, tem1, -muz0, sqrt(2/_pi))'

mat tem1 = J(1, `nofvars', 0)
mat tem2 = I(`nofvars')
mat tem3 = tem2*(-betahat)

mat Gbar3 = tem1 \ tem2 \ tem1 \ tem3 \ muz1
* mat Gbar3 = (0, 1, 0, -betahat, -muz1)' /* */

gsort -`useobs' /* so that the useable observations are on the top portion */
forvalues jj = 1/`nofobs' {

   mat gi  = (`ye2'[`jj'] \ `zeye'[`jj'] \ `ze2'[`jj'] \ `zeye2'[`jj'] \ `ze2ye'[`jj'] \ `ye3'[`jj'])

     /* the R_i(mu) vector; (k+1)*2X1 for each obs */
   mat Ri = J((`nofvars'+1)*2, 1, 0) /* a column vector */
   mat Ri[1,1] = `ye'[`jj']
   forvalues kk = 1/`nofvars' {
        mat Ri[1+`kk',1] = `x`kk'ye'[`jj']
   }


   mat Ri[1+`nofvars'+1, 1] = `ze'[`jj']
   forvalues kk = 1/`nofvars' {
        mat Ri[1+`nofvars'+ 1 + `kk', 1] = `x`kk'ze'[`jj']
   }

     /* the ph_mu_i matrix */

   tempname tempm3
   mat `tempm3' = idenI2 # XX1
   if "`nosym'" == "" {
        mat `tempm3' = 0.5*(`tempm3' + `tempm3'') /* force it to be symmetric */
   }
   mat phii = syminv(`tempm3')*Ri


   mat phimat = -(`nofobs')*(asymcov)*C'*syminv(omega)*(gi-gb+Gbar*phii)
   mat phibeta1 = -phimat[1,....]
   mat phisigu5 = -phimat[5,....]

   mat mystry = mystry+ (phii', phibeta1, phisigu5)'*(phii', phibeta1, phisigu5)/((`nofobs')^2)

   mat mystry2 = mystry2 + (phii', phibeta1)'*(phii', phibeta1)/((`nofobs')^2)

}





mat alphvar = (Gbar2'*(mystry)*Gbar2)
scalar alphstd = sqrt(alphvar[1,1])

mat xvan = (Gbar3'*(mystry2)*Gbar3)

forvalues i = 1/`nofvars' {
 scalar xstd`i' = sqrt(xvan[`i',`i'])
}


******* get sigu2 and sigv2 **********

scalar sigmau2 = (sigu2)
scalar sigmav2 = (sigv)^2
* scalar u2std   = sqrt((ustd)^2*(2*sigu)^2)

******** degrees of freedoom *********
local nofall = `nofvars' + 4 /* 4: misvar, constant, sigma_u, sigma_v */
local dof = `nofobs' - `nofall'

******** print out results ***********

di " "
di "Number of observations: " `nofobs'
di " "
di "variable: coeff. (std.err.) t = t value, P>|t| = p value."
di " "

di "`misvar': " betahat " (" betastd "),  t= " betahat/betastd ", and P>|t| = " 2*ttail(`dof', abs(betahat/betastd)) "."

tokenize `okvars'
local ii = 1
while "`1'" ~= "" {
  di "`1': " gamma`ii' " (" xstd`ii' "), t = " gamma`ii'/xstd`ii' ", and P>|t| = " 2*ttail(`dof', abs(gamma`ii'/xstd`ii')) "."
  local ii = `ii' + 1
  mac shift
}

di "_cons: " alphahat " (" alphstd "), t = " alphahat/alphstd ", and P>|t| = " 2*ttail(`dof', abs(alphahat/alphstd)) "."

di "sigma_u^2: " sigmau2 " (" u2std "), t = " sigmau2/u2std ", and P>|t| = " 2*ttail(`dof', abs(sigmau2/u2std)) "."
di "sigma_v^2: " sigmav2 " (" v2std "), t = " sigmav2/v2std ", and P>|t| = " 2*ttail(`dof', abs(sigmav2/v2std)) "."


end
